import ResizeObserver from 'resize-observer-polyfill';
require('intersection-observer');

(($, { behaviors }, loadImage, loadedImage) => {
  behaviors.imageSizesBehavior = {
    attach(context) {
      $(context).once('image-sizes').each(function() {
        const $imgs = $('img.image-sizes:not(.load-always)', this);
        const $loadImgs = $('img.image-sizes.load-always', this);
        $imgs.get().map(v => {
          v = $(v);
          if (!v.data('image-sizes')) {
            v.data('image-sizes', JSON.parse(v.attr('data-src')));
          }
          v.on('image:size-check', function() {
            imageSizesSetUrl(v.get(0));
          })
        })

        $loadImgs.get().map((v) => $(v).trigger('image:size-check'));

        let resizeObserver = new ResizeObserver((entries) => {
          entries.map((v) => $(v.target).find('img.image-sizes').trigger('image:size-check'))
        });

        $loadImgs.map((i,v) => resizeObserver.observe($(v).parent().get(0)));

        let observer = new IntersectionObserver((entries, imageObserver) => {
          entries.map((entry) => {
            if (entry.isIntersecting) {
              imageObserver.unobserve(entry.target);
              $(entry.target).trigger('image:size-check');
              resizeObserver.observe($(entry.target).parent().get(0))
            }
          })
        });
        $imgs.get().map(v => observer.observe(v));
      })
    }
  }
  function setImage(ele, src, width, height) {
    ele.src = src;
    $(ele).attr('width', width);
    $(ele).attr('height', height);
    $(ele).trigger('image:size-checked');
  }
  function imageSizesSetUrl(ele) {
    var width = $(ele).parent().width();
    var conf = $(ele).data('image-sizes');
    var newUrl = false;
    let urls = Object.keys(conf).filter((v) => {
      if (v >= width) {
        return v;
      }
    }).map(v => conf[v]);

    if (urls.length > 0) {
      newUrl = urls[0];
    }

    if (newUrl == false) {
      newUrl = $(ele).attr('data-src-fallback');
    }
    if (newUrl !== ele.src) {
      if (loadedImage.has(newUrl)) {
        const dim = loadedImage.get(newUrl);
        setImage(ele, newUrl, dim.width, dim.height);
        return;
      }
      if (loadImage.has(newUrl)) {
        var eles = loadImage.get(newUrl);
        eles.push(ele);
        $(ele).addClass('loading');
        return;
      } else {
        loadImage.set(newUrl, [ele]);
      }
      const img = new Image();
      $(ele).addClass('loading');
      $(img).on('load', function() {
        const eles = loadImage.get(this.src);
        eles.map(e => {
          $(e).removeClass('pre-load').removeClass('loading').addClass('fade-in-on-load');
          setImage(e,this.src, img.naturalWidth, img.naturalHeight)
          loadedImage.set(this.src, {
            width: img.naturalWidth,
            height: img.naturalHeight
          });
        })
      });
      img.src = newUrl;
    }
  }
})(jQuery, Drupal, new Map(), new Map());
